﻿using NAisParser;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;
using System.Threading.Tasks;

namespace Madart.OperationalUI.AIS
{
	public class AISParser : IDisposable
	{
		Parser parser = new Parser();
		/// <summary>
		/// Parses a string from the AIS stream to a message object
		/// </summary>
		/// <param name="message"> </param>
		/// <returns>An object of type MessageType123 if the message maps to a type 1, 2 or 3 AIS message. Messagetype 5 if the message maps to a type 5, and null if the method fails</returns>
		public object ParseMessage(string message)
		{
			if (message == null) return null;
			if (message.Length == 0) return null;

			int count = message.Split(',').Length - 1;
			if (count != 6) return null;
			if (message[0] != '!') return null;

			var result = parser.TryParse(message, out var aisResult);

			if (!result) return null;

			if (aisResult.Type == 1 || aisResult.Type == 2 || aisResult.Type == 3)
			{
				try
				{
					parser.TryParse(aisResult, out MessageType123 type123Result);

					return type123Result;
				}
				catch (ArgumentException e)
				{
					Debug.WriteLine(e.StackTrace);
				}
			}else if(aisResult.Type == 5)
			{
				try
				{
					parser.TryParse(aisResult, out MessageType5 type5Result);

					return type5Result;
				}
				catch (ArgumentException e)
				{
					Debug.WriteLine(e.StackTrace);
				}
			}
			
			return null;
		}

		public void Dispose()
		{
			parser = null;
		}
	}
}
