﻿using GeoAPI.Geometries;
using Madart.OperationalUI.Common.Extensions.DataAccess;
using Madart.OperationalUI.DataAccess.Models;
using Madart.OperationalUI.SignalR.Models;
using Madart.OperationalUI.WebApp.AIS;
using Madart.OperationalUI.WebApp.States;
using NAisParser;
using NetTopologySuite.Geometries;
using NUnit.Framework;
using System;
using System.Collections.Generic;
using System.Text;
using static Madart.OperationalUI.WebApp.AIS.AisMessageHandler;

namespace Madart.OperationalUI.Tests.ControllersAndHandlers
{
    [TestFixture]
    class AISMessageHandlerTests
    {

        VesselState vesselstate;
        VesselStateCollection vesselStateManager;
        AisMessageHandler handler;
		CurrentVesselState currentVesselState;

        [SetUp]
        public void Setup()
        {
            handler = new AisMessageHandler();
            vesselStateManager = new VesselStateCollection();
            MessageType123 message = new MessageType123(new byte(), new byte(), 1234, 0, 0.0, 30, false, 90, 90, 100.0, 0, 0, 0, false, 0);
            vesselstate = new VesselState().MapFrom(message);
			currentVesselState = new CurrentVesselState().MapFrom(vesselstate);
        }

        [TearDown]
        public void Cleanup()
        {
            vesselStateManager.ClearState();
        }

		//TESTS FOR currentState.legalMove()
		[Test]
        public void TryUpdateTrondheimToOslo()
        {
            IPoint positionTrondheim = new Point(10.3, 64.5);
            VesselState vesselAtTrondheim = vesselstate;
            vesselAtTrondheim.Position = positionTrondheim;
            vesselAtTrondheim.TimeReceived = DateTime.Now;
            CurrentVesselState currentVesselAtTrondheim = new CurrentVesselState().MapFrom(vesselAtTrondheim);
            handler.TryUpdate(vesselStateManager, currentVesselAtTrondheim, null);

            IPoint positionOslo = new Point(10.7, 59.9);
            VesselState vesselAtOslo = new VesselState();
            MessageType123 message = new MessageType123(new byte(), new byte(), 1234, 0, 0.0, 30, false, positionOslo.X, positionOslo.Y, 100.0, 0, 0, 0, false, 0);
            VesselStateExtensions.MapFrom(vesselAtOslo, message);
            vesselAtOslo.Position = positionOslo;
            vesselAtOslo.TimeReceived = DateTime.Now;
            CurrentVesselState currentVesselAtOslo = new CurrentVesselState().MapFrom(vesselAtOslo);
            bool legal = handler.TryUpdate(vesselStateManager, currentVesselAtOslo, currentVesselAtTrondheim);
            Assert.False(legal);
        }

        [Test]
        public void TryUpdateOneKm()
        {
            IPoint positionStart = new Point(10.687748, 59.869520);
            IPoint positionEnd = new Point(10.705557, 59.869497);

            VesselState vesselStart = vesselstate;
            vesselStart.Position = positionStart;
            vesselStart.TimeReceived = DateTime.Now;
            CurrentVesselState currentVesselStart = new CurrentVesselState().MapFrom(vesselStart);
            handler.TryUpdate(vesselStateManager, currentVesselStart, null);


            VesselState vesselEnd = new VesselState();
            MessageType123 message = new MessageType123(new byte(), new byte(), 1234, 0, 0.0, 30, false, positionEnd.X, positionEnd.Y, 100.0, 0, 0, 0, false, 0);
            vesselEnd.TimeReceived = DateTime.Now;
            VesselStateExtensions.MapFrom(vesselEnd, message);
            CurrentVesselState currentVesselEnd = new CurrentVesselState().MapFrom(vesselEnd);
            bool legal = handler.TryUpdate(vesselStateManager, currentVesselEnd, currentVesselStart);
            Assert.True(legal);
        }

        [Test]
        public void TryUpdateTenMeters()
        {
            IPoint positionStart = new Point(10.687069, 59.887123);
            IPoint positionEnd = new Point(10.686890, 59.887125);

            VesselState vesselStart = vesselstate;
            vesselStart.Position = positionStart;
            vesselStart.TimeReceived = DateTime.Now;
            CurrentVesselState currentVesselStart = new CurrentVesselState().MapFrom(vesselStart);
            handler.TryUpdate(vesselStateManager, currentVesselStart, null);

            VesselState vesselEnd = new VesselState();
            MessageType123 message = new MessageType123(new byte(), new byte(), 1234, 0, 0.0, 30, false, positionEnd.X, positionEnd.Y, 100.0, 0, 0, 0, false, 0);
            vesselEnd.TimeReceived = DateTime.Now;
            VesselStateExtensions.MapFrom(vesselEnd, message);
            CurrentVesselState currentVesselEnd = new CurrentVesselState().MapFrom(vesselEnd);
            bool legal = handler.TryUpdate(vesselStateManager, currentVesselEnd, currentVesselStart);
            Assert.False(legal);
        }
    }

}
