﻿using Madart.OperationalUI.Common.Extensions.DataAccess;
using Madart.OperationalUI.DataAccess.Models;
using Madart.OperationalUI.MadAPI.Models;
using Madart.OperationalUI.SignalR.Models;
using Madart.OperationalUI.WebApp.States;
using NAisParser;
using NUnit.Framework;
using System;
using System.Collections.Generic;
using System.Text;

namespace Madart.OperationalUI.Tests.ModelsAndStates
{
	class VesselStateManagerTests
	{
		VesselStateCollection vesselStateManager;
		MessageType123 message;
		VesselState vesselState;
		CurrentVesselState currentVesselState;
		[SetUp]
		public void Setup()
		{
			vesselStateManager = new VesselStateCollection();
			message = new MessageType123(new byte(), new byte(), 1234, 0, 0.0, 30, false, 90, 90, 100.0, 0, 0, 0, false, 0);
			vesselState = new VesselState().MapFrom(message);
			currentVesselState = new CurrentVesselState().MapFrom(vesselState);
		}

		[TearDown]
		public void Cleanup()
		{
			vesselStateManager.ClearState();
		}

		[Test]
		public void AddCurrentVesselStateEmptyList()
		{
			Assert.True(vesselStateManager.AddCurrentVesselState(currentVesselState));
		}

		[Test]
		public void AddCurrentVesselStateOccupiedList()
		{
			vesselStateManager.AddCurrentVesselState(currentVesselState);
			Assert.False(vesselStateManager.AddCurrentVesselState(currentVesselState));

		}

		[Test]
		public void UpdateDeviationNoKey()
		{
			Deviation deviation = new Deviation();
			Assert.False(vesselStateManager.UpdateDeviation(currentVesselState.MmsiNo, deviation));
		}
		[Test]
		public void UpdateDeviationNullDeviation()
		{
			Deviation deviation = null;
			Assert.False(vesselStateManager.UpdateDeviation(currentVesselState.MmsiNo, deviation));
		}
		[Test]
		public void UpdateDeviationOkDeviation()
		{
			Deviation deviation = new Deviation();
			vesselStateManager.AddCurrentVesselState(currentVesselState);
			Assert.True(vesselStateManager.UpdateDeviation(currentVesselState.MmsiNo, deviation));
		}

		//START HER HALLVARD
		[Test]
		public void UpdateNormalRouteNoKey()
		{
			var normalRoutes = new List<Tuple<int?, double?>>();
			for (int i = 0; i < 5; i++) {
				normalRoutes.Add(new Tuple<int?, double?>(i,i * 0.3));
			}

			Assert.False(vesselStateManager.UpdateNormalroute(0, normalRoutes));
		}

		[Test]
		public void UpdateNormalRouteNullList()
		{
			vesselStateManager.AddCurrentVesselState(currentVesselState);
			List<Tuple<int?, double?>> normalRoutes = null;

			Assert.False(vesselStateManager.UpdateNormalroute(currentVesselState.MmsiNo, normalRoutes));

		}


		[Test]
		public void UpdateNormalRouteEmptyList()
		{
			vesselStateManager.AddCurrentVesselState(currentVesselState);
			List<Tuple<int?, double?>> normalRoutes = new List<Tuple<int?, double?>>();

			Assert.False(vesselStateManager.UpdateNormalroute(currentVesselState.MmsiNo, normalRoutes));

		}

		[Test]
		public void UpdateNormalRouteLegalInput()
		{
			var normalRoutes = new List<Tuple<int?, double?>>();
			for (int i = 0; i < 5; i++)
			{
				normalRoutes.Add(new Tuple<int?, double?>(i, i * 0.3));
			}
			vesselStateManager.AddCurrentVesselState(currentVesselState);


			Assert.True(vesselStateManager.UpdateNormalroute(currentVesselState.MmsiNo, normalRoutes));
		}

		[Test]
		public void UpdateCurrentVesselStateNoMatch()
		{
			Assert.False(vesselStateManager.UpdateCurrentVesselState(currentVesselState));
		}

		[Test]
		public void UpdateCurrentVesselStateWithMatch()
		{
			vesselStateManager.AddCurrentVesselState(currentVesselState);
			Assert.True(vesselStateManager.UpdateCurrentVesselState(currentVesselState));
		}
	}
}
