﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Threading.Tasks;
using Madart.OperationalUI.AIS;
using Madart.OperationalUI.DataAccess.Repositories;
using Madart.OperationalUI.WebApp.States;
using Madart.OperationalUI.Hubs;
using Microsoft.AspNetCore.SignalR;

namespace Madart.OperationalUI.WebApp.AIS
{
	public class AisUpdateHandler
	{
		protected object DbSyncRoot = new object();
		private IVesselRepository Repository;

		public AisUpdateHandler(IVesselRepository vesselRepository)
		{
			Repository = vesselRepository;
		}
		/// <summary>
		/// This Task receives a MessageType5 object and handles it. This means updating our vesselstate manager and database.
		/// </summary>
		/// <param name="args">An AisUpdateEventArg, containing messagenumber, the MessageType5 parsed from a stream and time received</param>
		/// <param name="vesselStateManager">Interface connected to our vesselstateManager</param>
		/// <returns>Nothing as of yet, should later down the road return a status</returns>
		internal Task HandleUpdateAsync(AisUpdateEventArgs args, IVesselStateCollection vesselStateManager)
		{
			if (args == null || args.Message == null || args.Message.Mmsi == 0) return Task.FromResult<object>(null);
			var UpdateMessage = args.Message;
			vesselStateManager.UpdateDestination(UpdateMessage.Mmsi, UpdateMessage.Destination);
			vesselStateManager.UpdateTypeFiveShipType(UpdateMessage.Mmsi, UpdateMessage.ShipType.ToString());
			lock (DbSyncRoot)
			{
				Repository.UpdateVessel(UpdateMessage);
			}
			return Task.CompletedTask;
		}
	}
}
