﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Madart.OperationalUI.DataAccess.Repositories;
using Madart.OperationalUI.DataAccess.Models;
using Madart.OperationalUI.WebApp.States;
using Madart.OperationalUI.SignalR.Models;
using NetTopologySuite.Geometries;

namespace Madart.OperationalUI.WebApp.Handlers
{
    public interface IClientHandler
    {
        List<VesselState> GetHistory(int mmsi, int len = 400);
        Vessel GetVessel(int MmsiNo);
        List<Tuple<double?,NormalRoute>> GetNormalRoutesWithRank(int MmsiNo);
        GravityVector GetGravityVector(Point position, int normalRouteId);
        List<Location> GetDeppAndArrLocations(int normalrouteId);
    }
    public class ClientHandler : IClientHandler
    {
        IVesselRepository VesselRepository;
        INormalRouteRepository NormalRouteRepository;
        IVesselStateRepository VesselStateRepository;
        VesselStateCollection vesselStateManager;
        IGravityVectorRepository GravityVectorRepository;
        ILocationRepository LocationRepository;

        public ClientHandler(IVesselRepository vesselRepository, INormalRouteRepository normalRouteRepository, IVesselStateRepository vesselStateRepository, IGravityVectorRepository gravityVectorRepository, ILocationRepository locationRepository)
        {
            VesselRepository = vesselRepository;
            NormalRouteRepository = normalRouteRepository;
            VesselStateRepository = vesselStateRepository;
            vesselStateManager = new VesselStateCollection();
            GravityVectorRepository = gravityVectorRepository;
            LocationRepository = locationRepository;
        }

        public List<VesselState> GetHistory(int mmsi, int len = 400)
        {
            return VesselStateRepository.GetHistory(mmsi, len);
        }
		/// <summary>
		/// Fetches normal routes from DB that are classified to the vessel
		/// </summary>
		/// <param name="MmsiNo">MMSI number to the relevant vessel</param>
		/// <returns>A Tuple list of normal routes, and their corresponding rank</returns>
        public List<Tuple<double?, NormalRoute>> GetNormalRoutesWithRank(int MmsiNo)
        {
            var rankList = vesselStateManager.GetNormalRouteIdRankList(MmsiNo);
            if (rankList == null || rankList.Count == 0 || rankList[0].Item1 == -1) return null;
            List<int?> IDs = new List<int?>();
            foreach(var e in rankList)
            {
                IDs.Add(e.Item1);
            }
            List<NormalRoute> normalRoutes = NormalRouteRepository.GetNormalRoutes(IDs);
            List<Tuple<double?, NormalRoute>> RankedRoutes = new List<Tuple<double?, NormalRoute>>();
            for(int i=0; i<normalRoutes.Count; i++)
            {
                RankedRoutes.Add( new Tuple<double?, NormalRoute>(rankList[i].Item2, normalRoutes[i]));
            }
            
            return RankedRoutes;
        }

        public GravityVector GetGravityVector(Point position, int normalRouteId)
        {
            return GravityVectorRepository.GetGravityVector(position, normalRouteId);
        }

        public Vessel GetVessel(int MmsiNo)
        {
            return VesselRepository.GetVessel(MmsiNo);
        }

        public List<Location> GetDeppAndArrLocations(int normalrouteId)
        {
            return LocationRepository.GetDeppAndArrLocations(normalrouteId);
        }
    }
    
}
