﻿using GeoAPI.Geometries;
using Madart.OperationalUI.WebApp;
using Madart.OperationalUI.WebApp.Handlers;
using Microsoft.AspNetCore.SignalR;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Threading.Tasks;
using Madart.OperationalUI.SignalR.Models;
using Madart.OperationalUI.DataAccess.Models;
using NetTopologySuite.Geometries;
using Madart.OperationalUI.WebApp.States;

namespace Madart.OperationalUI.Hubs
{
	public interface IVesselHub
	{
	}

	public class VesselHub : Hub, IVesselHub
	{
		private readonly IVesselUpdater Updater;
		private readonly IClientHandler ClientHandler;

		public VesselHub(IVesselUpdater updater, IClientHandler clientHandler)
		{
			Updater = updater;
			ClientHandler = clientHandler;
		}


		public async Task sendHistory(int mmsi)
		{
			var history = ClientHandler.GetHistory(mmsi);
			history.Reverse();
			await Clients.Caller.SendAsync("vesselHistory", history);

		}

		public Task<List<CurrentVesselState>> SendAllCurrentVesselStates()
		{
			return Task.FromResult(new VesselStateCollection().GetAllCurrentVesselStates());
		}
		public Task<List<Warning>> SendAllCurrentWarnings()
		{
			return Task.FromResult(new WarningCollection().GetAllCurrentWarnings());
		}
		public  Task<CurrentVesselState> SendCurrentVesselState(int mmsi)
		{
			return Task.FromResult(new VesselStateCollection().GetCurrentVesselState(mmsi));
		}

		public Task<Vessel> SendVessel(int mmsi)
		{
			var vessel = ClientHandler.GetVessel(mmsi);
			return Task.FromResult(vessel);
		}

		public Task<GravityVector> SendGravityVector(double positionX, double positionY, int normalrouteId)
		{
			var pos = new Point(new Coordinate(positionX, positionY));
			pos.SRID = 4326;
			var gravityVector = ClientHandler.GetGravityVector(pos, normalrouteId);
			return Task.FromResult(gravityVector);
		}

		public Task<List<Madart.OperationalUI.DataAccess.Models.Location>> SendDeppAndArrLocations(int normalrouteId)
		{
			var listOfLocations = ClientHandler.GetDeppAndArrLocations(normalrouteId);
			return Task.FromResult(listOfLocations);
		}

		public Task<List<Tuple<double?, Madart.OperationalUI.DataAccess.Models.NormalRoute>>> SendNormalRoutes(int mmsi)
		{
			try
			{
				var normalRoutes = ClientHandler.GetNormalRoutesWithRank(mmsi);
				if (normalRoutes != null)
				{
					return Task.FromResult(normalRoutes);
				}
				else
				{
					return null;
				}
			}
			catch (Exception e)
			{
				Debug.WriteLine("Error while getting normal routes: " + e.Message);
				return null;
			}
		}
	}
}

