using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.HttpsPolicy;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Madart.OperationalUI.Hubs;
using Microsoft.Extensions.Hosting;
using Microsoft.EntityFrameworkCore;
using Madart.OperationalUI.WebApp;
using NetTopologySuite.IO;
using Microsoft.AspNetCore.SignalR.Protocol;
using Madart.OperationalUI.DataAccess.Repositories;
using Madart.OperationalUI.MadAPI;
using Madart.OperationalUI.WebApp.States;
using Madart.OperationalUI.WebApp.Handlers;
using Madart.OperationalUI.DataAccess;

namespace Madart.OperationalUI
{
	public class Startup
	{
		public static IServiceProvider __serviceProvider;
		public IConfiguration Configuration { get; }

		public Startup(IConfiguration configuration)
		{
			Configuration = configuration;
		}

		// This method gets called by the runtime. Use this method to add services to the container.
		public void ConfigureServices(IServiceCollection services)
		{
			services.Configure<CookiePolicyOptions>(options =>
			{
				// This lambda determines whether user consent for non-essential cookies is needed for a given request.
				options.CheckConsentNeeded = context => true;
				options.MinimumSameSitePolicy = SameSiteMode.None;
			});

			// This is where we add our services_ DB-context, SignalR, and different Transient or Singleton-timed objects
			services.AddDbContext<MadartDbContext>(
				options => options.UseNpgsql( Configuration.GetConnectionString("PostgreSQL"),
				npgsqlOptions => npgsqlOptions.UseNetTopologySuite())
				, ServiceLifetime.Transient);

			services.AddHostedService<VesselStateDatabaseCleaner>();
			services.AddMvc().SetCompatibilityVersion(CompatibilityVersion.Version_2_2);
			services.AddSingleton<IConfiguration>(Configuration);

			services.AddSignalR().AddJsonProtocol(options =>
			{
				options.PayloadSerializerSettings.ReferenceLoopHandling = Newtonsoft.Json.ReferenceLoopHandling.Ignore;
			});

			services.AddTransient<IVesselStateRepository, VesselStateRepository>();
			services.AddTransient<IVesselRepository, VesselRepository>();
			services.AddTransient<IGravityVectorRepository, GravityVectorRepository>();
			services.AddTransient<INormalRouteRepository, NormalRouteRepository>();
			services.AddTransient<ILocationRepository, LocationRepository>();


			services.AddSingleton<IHostedService, VesselUpdater>();
			services.AddSingleton<IVesselUpdater, VesselUpdater>();
			services.AddSingleton<IClientHandler, ClientHandler>();
			services.AddSingleton<IVesselStateCollection, VesselStateCollection>();


		}

		// This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
		public void Configure(IApplicationBuilder app, Microsoft.AspNetCore.Hosting.IHostingEnvironment env)
		{
			if (env.IsDevelopment())
			{
				app.UseDeveloperExceptionPage();
			}
			else
			{
				app.UseExceptionHandler("/Error");
				// The default HSTS value is 30 days. You may want to change this for production scenarios, see https://aka.ms/aspnetcore-hsts.
				app.UseHsts();
			}

			app.UseHttpsRedirection();
			app.UseStaticFiles();
			app.UseCookiePolicy();
			

			app.UseSignalR(routes =>
			{
				routes.MapHub<VesselHub>("/vesselHub");                
			});

			app.UseMvc();
		}
	}
}
