﻿// Please see documentation at https://docs.microsoft.com/aspnet/core/client-side/bundling-and-minification
// for details on configuring this project to bundle and minify static web assets.

// A layer group for all vessel history paths

var script = document.createElement("script");  // create a script DOM node
script.src = '/lib/jquery-ui-1.12.1/jquery-ui.min.js';  // set its src to the provided URL

document.head.appendChild(script);

var dictionaryOfVesselStates = {}; // {... isMuted}
var markersDictionary = {};
var modalStateDictionary = {}; // {PK: id, attr: isPinned, showingTrack, showingForecast, isMinimized}
var warningsDictionary = {}; 

var map = L.map('mapid', { zoomControl: false }).setView([59.8902265625, 10.75646953125], 5);
L.control.zoom({ position: 'topright' }).addTo(map);
L.control.scale({ position: 'topright' }).addTo(map);
var connection = new signalR.HubConnectionBuilder().withUrl("/vesselHub").build();
var streetmap = L.tileLayer('https://{s}.tile.osm.org/{z}/{x}/{y}.png', {
	attribution: '&copy; <a href="https://osm.org">OpenStreetMap</a> contributors.'
});
var seamap = L.tileLayer('https://opencache.statkart.no/gatekeeper/gk/gk.open_gmaps?layers=sjokartraster&zoom={z}&x={x}&y={y}', {
	attribution: '<a href="http://www.kartverket.no/">Kartverket</a>'
});
var darkMattermap = L.tileLayer('https://{s}.basemaps.cartocdn.com/dark_all/{z}/{x}/{y}{r}.png', {
	attribution: '&copy; <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors &copy; <a href="https://carto.com/attributions">CARTO</a>',
	subdomains: 'abcd'
});
map.addLayer(streetmap);
var activeLayer = streetmap;
var markersLayer = L.featureGroup().addTo(map);
var topZIndex = 1;
var listOfPinnedVessels = []; //mmsi of all the windows opened

var normalRouteLayerGroup = L.layerGroup([]);
var vesselHistoryLayerGroup = L.layerGroup([]);
var warningMarkerGroup = L.featureGroup([]);
var warningSectorLayerGroup = L.featureGroup([]);
var vesselMarkerClusterGroup = L.markerClusterGroup({
	maxClusterRadius: 45, disableClusteringAtZoom: 8, spiderfyOnMaxZoom: false, polygonOptions: {weight:1, color: 'black', dfillOpacity:0.15}
});

normalRouteLayerGroup.addTo(map);
vesselHistoryLayerGroup.addTo(map);
warningMarkerGroup.addTo(map);
warningSectorLayerGroup.addTo(map);
vesselMarkerClusterGroup.addTo(map);

$(document).ready(async function () {

	// On window load - fetch current vessels and warnings from server
	connection.start().then(() => {
		let promise1 = new Promise((resolve, reject) => {
			resolve(connection.invoke("SendAllCurrentVesselStates").catch(err => console.log(err)))
		});

		promise1.then((vessels) => {
			for (var i = 0; i < vessels.length; i++) {
				receiveVesselState(vessels[i]);
			}
		});
		let promise2 = new Promise((resolve, reject) => {
			resolve(connection.invoke("SendAllCurrentWarnings").catch(err => console.loge(err)))
		});
		promise2.then((warnings) => {
			for (var i = 0; i < warnings.length; i++) {
				receiveWarning(warnings[i]);
			}
		})
	});


	//The method that is called when a user clicks on a vessel
	vesselMarkerClusterGroup.on("click", async function (event) {
		var clickedMarker = event.layer;
		var mmsiNo = clickedMarker.options.mmsiNo;
		var vesselState = dictionaryOfVesselStates[mmsiNo];


		// check if already in modalStateDictrionary
		if (mmsiNo in modalStateDictionary) {
			if (modalStateDictionary[mmsiNo].isMinimized) {
				await maximizeModal(mmsiNo);
				modalStateDictionary.isMinimized = false;
			}
			var modalId = modalStateDictionary[mmsiNo].modalId;
			$("#" + modalId).css("zIndex", topZIndex);
			topZIndex++;
			return;
		}
		// Ask database for vessel data
		const vesselTask = await connection.invoke("SendVessel", mmsiNo).catch(err => console.log(err));

		//remove all unpinned modals
		for (const [key, value] of  Object.entries(modalStateDictionary)) {
			if (!value.isPinned) {
				closeModal(key);
				delete modalStateDictionary[key];
			}
		}
	
		// add to dictionary
		var modalState = { 'MmsiNo': mmsiNo, 'isPinned': false, 'showingTrack': false, 'showingForecast': false, 'isMinimized': false, 'modalId': '' };
		modalStateDictionary[mmsiNo] = modalState;
		// get vessel info from database.
		var vessel = await vesselTask;
		// create modal
		modalStateDictionary[mmsiNo].modalId = await createModal(modalState, vessel, vesselState);
	});

	//Function that is invoked by server with a new vesselState
	connection.on("broadcastUpdate", function (data) { receiveVesselState(data); });

	//Function that is invoked by the server after a user has asked for a vessels trail
	connection.on("vesselHistory", function (data) {
		coords = [];

		for (i = 0; i < data.length; i++) {
			var weight = 0;
			if (data[i].relDistStd == null) {
				weight = 1;
			} else {
				weight = convertDeviationToColor(data[i].relDistStd) / 10;
			}
			current = [data[i].position.y, data[i].position.x, weight]
			coords.push(current);
		}

		var path = createPath(coords);
		path.options.mmsi = data[0].mmsiNo;
		vesselHistoryLayerGroup.addLayer(path);
		path.bringToFront();
	});

	//Function that is invoked by the server when a new warning is broadcasted
	connection.on("VesselInDangerOfRunningAground", function (data) {
		receiveWarning(data)
	})
});



//Function that is called when a user clicks on a warning
$(document).on('click', '.warning-vessel', function () {
	vesselMMSI = $(this).attr('mmsi');
	var vessel = dictionaryOfVesselStates[vesselMMSI];
	map.setView([vessel.position.y, vessel.position.x], 12);
})

//OnClick functions for the popup windows - trail
$(document).on('click', '.show-track', function () {
	var modalId = $(this).closest('.modal-dialog').attr('id');
	var mmsiNo = sliceMmsi(modalId)
	if ($(this).is(":checked")) {
		connection.invoke("sendHistory", mmsiNo).catch(err => console.log(err));
		modalStateDictionary[mmsiNo].showingTrack = true;
	} else {
		modalStateDictionary[mmsiNo].showingTrack = false;
		removeHistory(mmsiNo);
	}  
});


// cont. OnClick functions for the popup windows - forecast

$(document).on('click', '.show-normalroutes', async function () {
	var modalId = $(this).closest('.modal-dialog').attr('id');
	var mmsiNo = sliceMmsi(modalId);
	var vessel = dictionaryOfVesselStates[mmsiNo];
	
	if ($(this).is(":checked")) {
		await getAndDrawNormalRoutes(mmsiNo, vessel);
	} else {
		modalStateDictionary[mmsiNo].showingForecast = false;
		removeNormalRoutes(mmsiNo);
	}
});

// cont. OnClick functions for the popup windows - disable warnings

$(document).on('click', '.disable-warnings', function () {
	var modalId = $(this).closest('.modal-dialog').attr('id');
	var mmsiNo = sliceMmsi(modalId)
	if ($(this).is(":checked")) { // box just got checked
		setMuteWarnings(mmsiNo, true);
		//dictionaryOfVesselStates[mmsiNo].isMuted = true;
	} else {
		setMuteWarnings(mmsiNo, false);
		//dictionaryOfVesselStates[mmsiNo].isMuted = false;
	}
});



// Called when modal "Show Forecast" is clicked, fetches normalroutes from server and draws them for user
async function getAndDrawNormalRoutes(mmsiNo, vessel) {
    var NormalRouteTuples = await connection.invoke("SendNormalRoutes", mmsiNo).catch(err => console.log(err));
    if (NormalRouteTuples != null) {
        modalStateDictionary[mmsiNo].showingForecast = true;
        var maxRank = Math.max.apply(Math, NormalRouteTuples.map(function(o) { return o.item1; }));
        var minRank = Math.min.apply(Math, NormalRouteTuples.map(function(o) { return o.item1; }));
        if (maxRank < 0.1) {
            maxRank = maxRank * 10;
        }
        if (minRank < 0.1) {
            minRank = minRank * 10;
        }
        if (NormalRouteTuples.length == 1) {
            NormalRouteTuples[0].normalized_rank = NormalRouteTuples[0].item1;
            drawNormalroute(NormalRouteTuples[0], vessel);
        }
        else {
            for (var e in NormalRouteTuples) {
                var rank = (NormalRouteTuples[e].item1 - minRank) / (maxRank - minRank);
                NormalRouteTuples[e].normalized_rank = rank;
                drawNormalroute(NormalRouteTuples[e], vessel);
            }
        }
    }
}


//Handles a new warning from the server, adds it to the warning lists and draws it on the map
async function receiveWarning(warning) {
	let mmsiNo = warning.mmsiNo;
	warningsDictionary[mmsiNo] = { 'warning': warning }; // Creates a warningsDictionary object {warning, listElementId}

	if (dictionaryOfVesselStates[mmsiNo].isMuted) { 
		return;
	}
	let warningListIdTask = addVesselToWarningList(warning); // starts an async call to create a warning list element. Returns id of the DOM
	createWarningSector(warning);
	createWarningMarker(warning);

	warningsDictionary[mmsiNo].listItemId = await warningListIdTask;
}


// Creates a sector for a new warning
function createWarningSector(warning) {
	var coords = [];
	for (var e in warning.sectorPolygon) {
		current = [warning.sectorPolygon[e].y, warning.sectorPolygon[e].x];
		coords.push(current);
	}
	var polygon = L.polygon(coords, { fillOpacity:0.08,color:'red', weight:0.2,mmsiNo: warning.mmsiNo }).addTo(warningSectorLayerGroup);
}

// This methods create a list item and appends it to the sidebar
async function addVesselToWarningList(warning) {
	var getVesselTask = connection.invoke("sendVessel", warning.mmsiNo).catch(err => console.log(err));
	var name = warning.mmsiNo;
	var imo = "Not known";

	var closestTimeToImpact;
	if (warning.coastPointTTI < warning.skerryPointTTI) {
		closestTimeToImpact = warning.coastPointTTI;
	} else {
		closestTimeToImpact = warning.skerryPointTTI;
	}

	var timeRecieved = Date.now();
	var warningListId = warning.mmsiNo + "-warningListElement";

	var vessel = await getVesselTask;

	if (vessel != null) {
		name = vessel.vesselName;
		imo = vessel.imoNo;
	}

	var html = `<div class="warning-vessel html-object new-list-element" id="` + warningListId + `" mmsi="` + warning.mmsiNo + `" data-sort="`+timeRecieved+`">
			<div class="warning-header">
				<h3 style="color:white; width: 70%; float: left; margin-bottom: 0 !important; font-size: 17px;">`+ name +`</h3>
				<button class="button-style warning-close" type="button" data-dismiss="modal" aria-hidden="true" style="line-height: 0.4;">&times;</button>
			</div>
			<div class="text-div-info">MMSI: `+ warning.mmsiNo + ` </div>
			<div class="text-div-info">IMO: `+ imo + `</div>
			<div class="text-div-info" id="TTI`+ warning.mmsiNo + `">Time to impact: ` + Number(closestTimeToImpact).toFixed(0) + ` sec</div>
		</div>`;

	$("#mySidebar").append(html);
	$("#mySidebar .warning-vessel").sort(function (a, b) {
		return parseInt(b.dataset.sort) - parseInt(a.dataset.sort);
	}).appendTo('#mySidebar');
	return warningListId;
}
// Creates a warning triangle on a skerry or coast point, with click- and mouseover functions
function createWarningMarker(warning) {
	var newLatLng;
	var timeToImpact;
	var distanceToImpact;
	if (warning.skerryPointTTI <= warning.coastPointTTI) {
		newLatLng = new L.LatLng(warning.closestSkerry.y, warning.closestSkerry.x);
		timeToImpact = warning.skerryPointTTI;
		distanceToImpact = warning.distanceToClosestSkerry;
	} else {
		newLatLng = new L.LatLng(warning.closestCoastPoint.y, warning.closestCoastPoint.x);
		timeToImpact = warning.coastPointTTI;
		distanceToImpact = warning.distanceToClosestCoastPoint;
	}
	var dangerIcon = L.icon({
		iconUrl: "/resources/WarningMarker.png",
		iconSize: [20, 20]
	});
	var popup = L.popup().setContent('<strong>Time to impact: ' + timeToImpact.toFixed(0) + 'sec </strong><br>Distance to impact: ' + distanceToImpact.toFixed(1) + 'm');
	var marker = L.marker(newLatLng, { icon: dangerIcon, rotationOrigin: "center center", warningMmsi: warning.mmsiNo }).bindPopup(popup).addTo(warningMarkerGroup); //courseOverGround/2 fordi???? it works : ^)

	marker.on('mouseover', function (e) {
		this.openPopup();
	}).on('mouseout', function (e) {
		this.closePopup();
	});
	marker.on('click', function () {
		map.setView(newLatLng, 12);
	});
}

//Clears warnings for a vessel, used when a vessel is muted
function removeWarningMarkersAndWarningSectors(mmsiNo) {
	if (mmsiNo in warningsDictionary) { // if there is an active warning marker
		$("#" + warningsDictionary[mmsiNo].listItemId).remove();
		warningsDictionary[mmsiNo].listItemId = null;

		warningMarkerGroup.getLayers().forEach(function (obj) {
			if (obj.options.warningMmsi == mmsiNo) {
				warningMarkerGroup.removeLayer(obj);
			}
		});

		warningSectorLayerGroup.getLayers().forEach(function (o) {
			if (o.options.mmsiNo == mmsiNo) {

				warningSectorLayerGroup.removeLayer(o);
			}
		});
	}
}


// The function that draws a modal for a vessel
async function createModal(modalState, vessel, vesselState) {
	var name = "Not known";
	var imo = "Not known";
	var disabled = "";
	var callsign = "Not known";
	var imageUrl = "/resources/shipPlaceholder.png"; 
	if (vessel != null) {
		name = vessel.vesselName;
		imo = vessel.imoNo;
		callsign = vessel.callSign;

	}

	if (typeof vesselState.normalRoutesIdRank == "undefined" || vesselState.normalRoutesIdRank == null) {
		disabled = "disabled";
	}
	var darkmodeInfo = "";
	var darkmodeBG = "";
	var darkmodeBody = "";
	if (activeLayer == darkMattermap) { // makes modal darkmode, if the darkmap is the active map
		darkmodeBody = "modal-body-dark-mode";
		darkmodeBG = "bg-dark-mode";
		darkmodeInfo = "info-dark-mode";
	}
	var modalId = vesselState.mmsiNo + "-modal";
	var speedOverGround = vesselState.speedOverGround / 10.0 + " knots";
	var courseOverGround = vesselState.courseOverGround + "&deg;";
	var showTrackCheckBox = modalState.showingTrack ? "checked" : "";
	var showRouteForecastCheckBox = modalState.showingForecast ? "checked" : "";
	var isMutedCheckBox = vesselState.isMuted ? "checked" : "";
	var isPinnedColor = modalState.isPinned ? "#837c7c" : "#fff";

	var htmltxt = `<div class="modal-dialog html-object" id="` + modalId + `" value="` + vesselState.mmsiNo + `"><div class="modal-content">
		<div class="modal-header `+ darkmodeBG + `"><div><span class="pin" style="color: ` + isPinnedColor + `"> &nabla; </span><h4 class="modal-title text-primary" style="color: white; float: right !important;" id="feature-title"> ` + name + ` </h4></div><div class="buttons"><button class="button-style modal-close" type="button" data-dismiss="modal" aria-hidden="true">&times;</button><button class="button-style modal-minimize"> &hellip; </button></div></div>
		<div class="modal-body `+ darkmodeBody + `" id="feature-info"><div class="info-container"><div class="info ` + darkmodeInfo +` info-left"><div class="text-container">
		<div class="text-div" align="left"><h6>MMSI: </h6></div><div class="text-div"><span> `+ vesselState.mmsiNo + ` </span></div>
		<div class="text-div" align="left"><h6>IMO: </h6></div><div class="text-div"><span> `+ imo + ` </span></div>
		<div class="text-div" align="left"><h6>Callsign: </h6></div><div class="text-div"><span> `+ callsign + ` </span></div>
		<div class="text-div"><h6>Speed: </h6></div><div class="text-div" id="` + vesselState.mmsiNo + `-sog"> ` + speedOverGround + ` </div>
		<div class="text-div"><h6>Course: </h6></div><div class="text-div" id="` + vesselState.mmsiNo + `-cog"> ` + courseOverGround + ` </div></div></div>
		<div class="info `+ darkmodeInfo + ` info-right"><img id="` + vesselState.mmsiNo + `-img" src="` + imageUrl + `" onerror="this.style.display = 'none'" alt="Image unavailable :'(" width="100%" height="100%"></div></div>
			<div class="button-container extra-container">
				<div class="custom-control custom-checkbox">
					<input type="checkbox" class="custom-control-input show-track" id="`+ vesselState.mmsiNo + `-showTrack" ` + showTrackCheckBox + `>
					<label class="custom-control-label" for="`+ vesselState.mmsiNo + `-showTrack">Show track</label>
				</div>
				<div class="custom-control custom-checkbox">
					<input type="checkbox" class="custom-control-input show-normalroutes" id="`+ vesselState.mmsiNo + `-showForecast" ` + showRouteForecastCheckBox + ` ` + disabled + `>
					<label class="custom-control-label" for="`+ vesselState.mmsiNo + `-showForecast">Route forecast</label>
				</div>
				<div class="custom-control custom-checkbox">
					<input type="checkbox" class="custom-control-input disable-warnings" id="`+ vesselState.mmsiNo + `-disableWarnings" ` + isMutedCheckBox + `>
					<label class="custom-control-label" for="`+ vesselState.mmsiNo + `-disableWarnings">Disable warnings</label>
				</div></div>
			<div class="button-container">
			<button type="button" class="btn button-blue button-zoom-to `+ darkmodeBG +`">Zoom To</button>
			</div></div></div></div>`;

	$("main").append(htmltxt);
	$("#" + modalId).css({ "zIndex": topZIndex });
	$("#" + modalId).data("vesseldata", vesselState);
	$("#" + modalId).draggable({ cancel: "div.text-div, h4.modal-title" });
	topZIndex++;

	
	var img = new Image();
	var imageFromWeb = "http://ship-info.com/vessels/" + imo + ".jpg";
	img.src = imageFromWeb;
	img.onload = function ()
	{ 
		var heightOfImg = this.height;
		if (heightOfImg != 0) {
			$("#" + vesselState.mmsiNo + "-img").attr("src", imageFromWeb);
		}
		
	}

	return modalId;
}

// Method that creates a minimized modal 
async function createMinimizedModal(mmsiNo, name) {
	modalId = mmsiNo + "-minimized";

	var htmltxt = `<div class="minimized html-object modal-maximize" id="` + modalId + `" value="` + mmsiNo + `">
		<h4 class="modal-title text-primary" style="color: white; float: left !important; font-size: 16px !important; line-height: 2.5 !important; width: 150px; overflow: hidden; text-overflow: ellipsis; white-space: nowrap;" id="feature-title">`+ name + `</h4>
		<div class="buttons"><button class="modal-close button-style" type="button" data-dismiss="modal" aria-hidden="true">&times;</button></div>
	</div>`;
	$("#" + modalId).val(mmsiNo);
	$("#minimized-container").append(htmltxt);
	return modalId;
}



// The method for handling a new vesselstate
// If the vessel already exists on the map, it will update its position.If not, it will add a new boatmarker on the map
function receiveVesselState(vessel) {
	if (vessel.mmsiNo in dictionaryOfVesselStates) {
		vessel.isMuted = dictionaryOfVesselStates[vessel.mmsiNo].isMuted;
	}
	else {
		vessel.isMuted = false;
	}
	var oldVessel = dictionaryOfVesselStates[vessel.mmsiNo]
	dictionaryOfVesselStates[vessel.mmsiNo] = vessel;
	if (markersDictionary[vessel.mmsiNo] != null) { // vessel is already on the map
		updateIcon(vessel);
		vesselHistoryLayerGroup.getLayers().forEach(async function (obj) { // vesselhistory is showing
			if (obj.options.mmsi == vessel.mmsiNo) {
				var weight = 0;
				if (vessel.relDistStd == null) {
					weight = 1;
				} else {
					weight = convertDeviationToColor(vessel.relDistStd) / 10;
				}
				if (vessel.newMessage) {
					obj.addLatLng([vessel.position.y, vessel.position.x, weight]);
				} else {
					var latlngs = obj.getLatLngs();
					latlngs[latlngs.length - 1].alt=weight;
					obj.setLatLngs(latlngs);

				}

				

			}
		});

		// updates realtime info in modal-window
		var myEle = document.getElementById("" + vessel.mmsiNo + "-sog");
		if (myEle) {
			$("#" + vessel.mmsiNo + "-sog").html(vessel.speedOverGround / 10.0 + " knots");
			$("#" + vessel.mmsiNo + "-cog").html(vessel.courseOverGround + "&deg;");
		}

		if (vessel.newMessage) {
			removeWarningMarkersAndWarningSectors(vessel.mmsiNo);
		} else {
			
			if (modalStateDictionary[vessel.mmsiNo] != null) {
				if (modalStateDictionary[vessel.mmsiNo].showingForecast) {
					if (!isEqual(vessel.normalRoutesIdRank,oldVessel.normalRoutesIdRank)) {

						removeNormalRoutes(vessel.mmsiNo);
						getAndDrawNormalRoutes(vessel.mmsiNo, vessel);

					}
				}
			}
		}
		return;
	}

	updateIcon(vessel, true);

	if (vesselMarkerClusterGroup.getChildCount < 10) {
		vesselMarkerClusterGroup.options.disableClusteringAtZoom = 1;
	}
}
// Function for updating a vessels icon
async function updateIcon(vessel, isNew = false) {
	let newIcon = getIcon(vessel);
	var newLatLng = new L.LatLng(vessel.position.y, vessel.position.x);
	if (isNew) {
		var marker = L.marker(newLatLng, { icon: await newIcon, rotationOrigin: "center center", rotationAngle: vessel.courseOverGround, mmsiNo: vessel.mmsiNo }); //courseOverGround/2 fordi???? it works : ^)
		vesselMarkerClusterGroup.addLayer(marker);
		markersDictionary[vessel.mmsiNo] = marker;
	}
	else {
		if (vessel.newMessage == false) {
			let icon = await newIcon;
			markersDictionary[vessel.mmsiNo].setIcon(icon);
		}
		markersDictionary[vessel.mmsiNo].setRotationAngle(vessel.courseOverGround);
		markersDictionary[vessel.mmsiNo].setLatLng(newLatLng).update();
		
	}
}

// Fetches corresponding vesselicon given speed and deviation
function getIcon(vessel) {	
	let url = "";
	let iconSize = [12, 24];
	if (vessel.speedOverGround == 0) {
		url = "/resources/VesselStationary.png";
		iconSize = [10, 10];
	} else if (vessel.isMuted) {
		url = "/resources/VesselGray.png";
	} else if (vessel.relDistStd == null) {
		url = "/resources/VesselStandard.png";
	} else if (vessel.relDistStd <= 3) {
		url = "/resources/VesselGreen.png";
	} else if (vessel.relDistStd <= 6) {
		url = "/resources/VesselYellow.png";
	} else {
		url = "/resources/VesselRed.png";
	}
	var vesselIcon = L.icon({
		iconUrl: url,
		iconSize: iconSize
	});
	return vesselIcon;
}

// Draws a normal route and maps a pop-up function to show Dep/Arr and ETA
async function drawNormalroute(tuple, vesselState) { // This needs a vesselstate for mmsi and gravityvector db call
	var list = tuple.item2.normalRouteGeometry.coordinates;

	var normalrouteId = tuple.item2.normalRouteId;
	var coords = [];
	for (var e in list) {
		current = [list[e].x, list[e].y]
		coords.push(current);
	}
	var normalroute1 = [{
		"type": "LineString",
		"coordinates": coords
	}];


	var locations = await connection.invoke("SendDeppAndArrLocations", normalrouteId).catch(err => console.log(err));

	var departureLocation;
	var arrivalLocation;
	if (locations[0] != null && locations[1] != null) {
		departureLocation = locations[0].locationNameNor;
		arrivalLocation = locations[1].locationNameNor;
	}

	var gravityVector = await connection.invoke("SendGravityVector", vesselState.position.coordinate.x, vesselState.position.coordinate.y, normalrouteId).catch(err => console.log(err));
	var layer = L.geoJSON(normalroute1, { style: normalRouteStyle(tuple), normalrouteId: normalrouteId, mmsiNo: vesselState.mmsiNo }).addTo(normalRouteLayerGroup).bindPopup('<strong> Voyage: ' + departureLocation + ' - ' + arrivalLocation + ' </strong><br>ETA: ' + secondsToHms(gravityVector.eta));
	layer.on('mouseover', function (e) {
		var popup = e.target.getPopup();
		popup.setLatLng(e.latlng).openOn(map);
		var mouseoverLayer = e.target;
		mouseoverLayer.setStyle({
			weight: 5
		});
	}).on('mouseout', function (e) {
		var mouseoverLayer = e.target;
		mouseoverLayer.setStyle({
			weight: 3
		});

	});
	
	normalRouteLayerGroup.addLayer(layer);
}
// Method for muting a vessel that isn't interesting to a user
function setMuteWarnings(mmsiNo, isMuted) { // (int mmsiNo, bool, diconaryOfVesselStates)
	if (mmsiNo in dictionaryOfVesselStates) {
		dictionaryOfVesselStates[mmsiNo].isMuted = isMuted;
		var checkbox = $("#" + mmsiNo + "-disableWarnings");
		if (typeof checkbox[0] != 'undefined') checkbox[0].checked = isMuted;
		var vessel = dictionaryOfVesselStates[mmsiNo];
		updateIcon(vessel);
		if (isMuted) {
			// remove warning markings on map
			removeWarningMarkersAndWarningSectors(mmsiNo);
		}
		else {
			// get possible warning from warningsDictionary
			var warning = warningsDictionary[mmsiNo].warning
			receiveWarning(warning);
		}
	}
}
// Small function for styling the warning list if an element has been mouse-overed
$(document).on('mouseover', '.warning-vessel', function () {
	$(this).removeClass('new-list-element');
})

// Zooms the map to current vessel
$(document).on('click', '.button-zoom-to', function () {
	var modalId = $(this).closest('.modal-dialog').attr('id');
	var mmsiNo = sliceMmsi(modalId)
	var vesselInfo = dictionaryOfVesselStates[mmsiNo];
	map.setView([vesselInfo.position.y, vesselInfo.position.x], 12);

});


// Sets a modal to pinned - modal will persist through clicks
$(document).on('click', '.pin', function () {
	var modalId = $(this).closest('.modal-dialog').attr('id');
	var mmsiNo = sliceMmsi(modalId)
	var isPinned = !modalStateDictionary[mmsiNo].isPinned;
	if (isPinned) {
		$(this).css({ "color": '#837c7c' });
	}
	else {
		$(this).css({ "color": '#fff' });
	}
	modalStateDictionary[mmsiNo].isPinned = isPinned;
});

// Closes the modal
$(document).on('click', '.modal-close', function (e) {
	var modalId = $(this).closest('.modal-dialog').attr('id');
	if (modalId == null) {
		modalId = $(this).closest('.minimized').attr('value');
	}
	var mmsiNo = sliceMmsi(modalId)

	closeModal(mmsiNo);
	e.stopPropagation();
});

// Closes a warninglist element and mutes it till further action is done
$(document).on('click', '.warning-close', function (e) {
	var mmsiNo = $(this).closest('.warning-vessel').attr('mmsi');
	var element = this.closest('.warning-vessel');


	element.remove();
	//dictionaryOfVesselStates[mmsiNo].isMuted = true;
	setMuteWarnings(mmsiNo, true);
	e.stopPropagation();
});

// Minimizes a modal - acts as pinned
$(document).on('click', '.modal-minimize', async function (e) {
	var modalId = $(this).closest('.modal-dialog').attr('id');
	var mmsiNo = sliceMmsi(modalId);
	var name = $(this).closest('.modal-dialog').find('.modal-title').text();
	modalIdTask = createMinimizedModal(mmsiNo, name);
	closeModal(mmsiNo, true);
	modalStateDictionary[mmsiNo].isMinimized = true;
	modalStateDictionary[mmsiNo].isPinned = true;
	modalId = await modalIdTask;
	modalStateDictionary[mmsiNo].modalId = modalId;
});

// Maximizes a minimized modal
$(document).on('click', '.modal-maximize', function (e) {
	var modalId = $(this).closest('.minimized').attr('id');
	var mmsiNo = sliceMmsi(modalId)
	maximizeModal(mmsiNo);
});

// Function that minimizes or maximizes the sidebar
$(document).on('click', '#sidebarButton', function () {
	if ($('#mySidebar').attr('value') == 1) {
		$('#mySidebar').width(0);
		$('#mySidebar').attr('value', 0);
	} else {
		$('#mySidebar').width('10%');
		$('#mySidebar').attr('value', 1);
	}
});

// Tile-changing functions - streetmap
$(document).on('click', '#streetmap-button', function () {
	map.removeLayer(activeLayer);
	map.addLayer(streetmap);
	activeLayer = streetmap;
});

// Tile-changing functions - seamap

$(document).on('click', '#seamap-button', function () {
	map.removeLayer(activeLayer);
	map.addLayer(seamap);
	activeLayer = seamap;
});

// Tile-changing functions - darkmap

$(document).on('click', '#darkmap-button', function () {
	map.removeLayer(activeLayer);
	map.addLayer(darkMattermap);
	activeLayer = darkMattermap;

});

// Function that clears unpinned modals at map-click
map.on('click', function (e) {
	for (const [key, value] of Object.entries(modalStateDictionary)) {
		if (!value.isPinned) {
			closeModal(key);
		}
	}
});

//Standard time-converting method
function secondsToHms(d) {
	d = Number(d);
	var h = Math.floor(d / 3600);
	var m = Math.floor(d % 3600 / 60);
	var s = Math.floor(d % 3600 % 60);

	var hDisplay = h > 0 ? h + (h == 1 ? " hour, " : " hours, ") : "";
	var mDisplay = m > 0 ? m + (m == 1 ? " minute, " : " minutes, ") : "";
	var sDisplay = s > 0 ? s + (s == 1 ? " second" : " seconds") : "";
	return hDisplay + mDisplay + sDisplay;
}
// Fetches style for the normal route, given rank
function normalRouteStyle(normalRoute) {
	var col = getColor(normalRoute.normalized_rank);
	return {
		weight: 3,
		opacity: 1,
		color: col,
		dashArray: "5 4",

	};
}
// Sets the color for a normal route, given rank
function getColor(rank) {
	//if (rank < 0.1) { rank = (rank+0.05) * 10; }
	return hslToHex(120 * rank, 100, 50);

}




// Method for fetching color, given devation value
function convertDeviationToColor(stdDev) {
	var maxDev = 8;
	var relDeviation = Math.min(stdDev, maxDev) / maxDev;
	return relDeviation;
}
//Clears history
function removeHistory(mmsiNo) {
	vesselHistoryLayerGroup.getLayers().forEach(async function (obj) {
		if (obj.options.mmsi == mmsiNo) {
			obj.remove();
		}
	});
}
// Remove normal routes
function removeNormalRoutes(mmsiNo) {
	normalRouteLayerGroup.getLayers().forEach(async function (obj) {
		if (obj.options.mmsiNo == mmsiNo) {
			obj.remove();
		}
	});
}
// Help method for closing modals
function closeModal(mmsiNo, minimized = false) {
	var modalId = modalStateDictionary[mmsiNo].modalId;
	if (!minimized) {
		removeHistory(mmsiNo);
		removeNormalRoutes(mmsiNo);
		delete modalStateDictionary[mmsiNo];
	}
	$("#" + modalId).remove();
}

// Help method for slicing mmsi from class-names
function sliceMmsi(modalId) {
	var index = modalId.indexOf('-');
	if (index != -1) {
		return modalId.slice(0, index);
	}
	return modalId;
}
// Help method for maximizing a modal
async function maximizeModal(mmsiNo) {
	const vesselTask = connection.invoke("SendVessel", mmsiNo).catch(err => console.log(err));
	var modalState = modalStateDictionary[mmsiNo];
	$("#" + modalState.modalId).remove();
	var vesselState = dictionaryOfVesselStates[mmsiNo];
	var vessel = await vesselTask;
	var modalId = await createModal(modalState, vessel, vesselState);
	modalStateDictionary[mmsiNo].isMinimized = false;
	modalStateDictionary[mmsiNo].modalId = modalId;
}


// Help method for equality-check between normal-route lists. 
function isEqual(newRoutes, oldRoutes) {
	if (newRoutes.length != oldRoutes.length) return false;
	for (var i = 0; i < newRoutes.length; i++) {
		if (newRoutes[i].item1 != oldRoutes[i].item1) return false;
	}
	return true;
}
// Standard method for convert HSL-values to hex
function hslToHex(h, s, l) {
	h /= 360;
	s /= 100;
	l /= 100;
	let r, g, b;
	if (s === 0) {
		r = g = b = l; // achromatic
	} else {
		const hue2rgb = (p, q, t) => {
			if (t < 0) t += 1;
			if (t > 1) t -= 1;
			if (t < 1 / 6) return p + (q - p) * 6 * t;
			if (t < 1 / 2) return q;
			if (t < 2 / 3) return p + (q - p) * (2 / 3 - t) * 6;
			return p;
		};
		const q = l < 0.5 ? l * (1 + s) : l + s - l * s;
		const p = 2 * l - q;
		r = hue2rgb(p, q, h + 1 / 3);
		g = hue2rgb(p, q, h);
		b = hue2rgb(p, q, h - 1 / 3);
	}
	const toHex = x => {
		const hex = Math.round(x * 255).toString(16);
		return hex.length === 1 ? '0' + hex : hex;
	};
	return `#${toHex(r)}${toHex(g)}${toHex(b)}`;
}
// Help method for coloring the trail given weight
function createPath(path) {
	var hotlineLayer = L.hotline(path, {
		min: 0,
		max: 1,
		palette: {
			0.0: 'green',
			0.075: 'yellow',
			0.1: 'red',
			1: 'black'
		},
		weight: 3,
		outlineWidth: 1,
		mmsi: 0
	});

	return hotlineLayer;
}
//Brings current modal to front on click
$(document).on('mousedown', '.modal-dialog', function () {
	$(this).css({ "zIndex": topZIndex });
	topZIndex++;
});
// Currently unused method for fetching color given speed
function convertSpeedToColor(speed) {
	var topSpeed = 15;
	var speed2 = Math.min(speed / topSpeed, topSpeed) / topSpeed;
	return speed2;
}

function sleep(ms) {
	return new Promise(resolve => setTimeout(resolve, ms));
}
